/* ScummVM - Graphic Adventure Engine
 *
 * ScummVM is the legal property of its developers, whose names
 * are too numerous to list here. Please refer to the COPYRIGHT
 * file distributed with this source distribution.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/trunk/backends/platform/iphone/osys_events.cpp $
 * $Id: osys_events.cpp 42479 2009-07-14 13:52:11Z vinterstum $
 *
 */

#include <sys/time.h>
#include <unistd.h>
#include "osys_main.h"
#include "define_gob.h"

#import <UIKit/UIKit.h>

//////////////////
//---GAME PARAMS
//LVL (GENERAL)
extern const char* niveau;
extern bool bPauseActive;
extern int idLvLCourant;
//LVL (SIMULATION)
extern bool bSimulation;
extern bool bLvLOneLoaded; 
extern bool bSimulCode;
extern int idLvLToLoad;
extern int etat;
extern uint16 m_codeLvLAscii[21][7];
extern Common::KeyCode m_codeLvLKeyCode[21][7];
//---END
////////////////

bool OSystem_IPHONE::pollEvent(Common::Event &event)
{
	//printf("pollEvent()\n");
	
	long curTime = getMillis();

	if (_timerCallback && (curTime >= _timerCallbackNext)) 
	{
		_timerCallback(_timerCallbackTimer);
		_timerCallbackNext = curTime + _timerCallbackTimer;
	}

	if (_needEventRestPeriod)
	{
		// Workaround: Some engines can't handle mouse-down and mouse-up events
		// appearing right after each other, without a call returning no input in between.
		_needEventRestPeriod = false;
		return false;
	}

	if (_queuedInputEvent.type != (Common::EventType)0) 
	{
		event = _queuedInputEvent;
		_queuedInputEvent.type = (Common::EventType)0;
		return true;
	}
	
	//-----------------------
	//--------SIMULATION HACK
	//-----------------------
	
	//PRE SIMULATION HACK
	preSimulation();
	
	//WAIT FRAMES BETWEEN ACTIONS (SIMUL HACK)
	if( bLoop != -1 &&  bLoop < 7 )
	{ bLoop++; return false; }
	else if( bLoop == 7 )
	{ etat++; bLoop = -1; }
	//END
	
	//WAIT FRAMES BETWEEN ACTIONS (SIMUL HACK)
	if( bLoop2 != -1 &&  bLoop2 < 12 )
	{ bLoop2++; return false; }
	else if( bLoop2 == 12 )
	{ etat++; bLoop2 = -1; }
	//END
	
	//SIMULATION HACK
	switch((InputSimulation)etat)
	{
		case kEscDown:
			if( cptEsc >= 8 && !bSimulCode )
			{
				return false;
			}
			event.type = Common::EVENT_KEYDOWN;
			event.kbd.flags = 0;
			event.kbd.keycode = Common::KEYCODE_ESCAPE;
			event.kbd.ascii  = Common::ASCII_ESCAPE;
			
			etat++;
			return true;
		case kEscUp:			
			if( cptEsc < 8 && !bSimulCode )
			{
				etat = -1;
				bLoop = 0;
				cptEsc++;
			}
			else if( cptEsc >= 8 && !bSimulCode )
			{
				return false;
			}
			else if( cptEsc >= 8 && bSimulCode )
			{
				etat++;
				cptEsc = 0;
			}
			event.type = Common::EVENT_KEYUP;
			event.kbd.flags = 0;
			event.kbd.keycode =  Common::KEYCODE_ESCAPE;
			event.kbd.ascii =  Common::ASCII_ESCAPE;
	
			return true;
		case kMouseMove:	
			event.type = Common::EVENT_MOUSEMOVE;
			event.mouse.x = 120.0f;
			event.mouse.y = 110.0f;
			
			etat++;
			return true;
		case kClickDown:			
			event.type = Common::EVENT_LBUTTONDOWN;
			event.mouse.x = 120.0f;
			event.mouse.y = 110.0f;
			
			bLoop = 0;
			return true;
		case kClickUp:
			event.type = Common::EVENT_LBUTTONUP;
			event.mouse.x = 120.0f;
			event.mouse.y = 110.0f;
			
			bLoop2 = 0;	
			return true;
		case kKeyDown:
			event.type = Common::EVENT_KEYDOWN;
			event.kbd.flags = 0;
			event.kbd.keycode = m_codeLvLKeyCode[idLvLToLoad - 2][cpt];
			event.kbd.ascii  = m_codeLvLAscii[idLvLToLoad - 2][cpt];
			
			etat++;
			return true;
		case kKeyUp:				
			event.type = Common::EVENT_KEYUP;
			event.kbd.flags = 0;
			event.kbd.keycode = m_codeLvLKeyCode[idLvLToLoad - 2][cpt];
			event.kbd.ascii  = m_codeLvLAscii[idLvLToLoad - 2][cpt];
			
			if( cpt < 6 )
			{ etat--; cpt++; }
			else 
				etat++;
			return true;
		case kEnterDown:			
			event.type = Common::EVENT_KEYDOWN;
			event.kbd.flags = 0;
			event.kbd.keycode = Common::KEYCODE_RETURN;
			event.kbd.ascii  = Common::ASCII_RETURN;
			
			etat++;
			return true;
		case kEnterUp:
			event.type = Common::EVENT_KEYUP;
			event.kbd.flags = 0;
			event.kbd.keycode = Common::KEYCODE_RETURN;
			event.kbd.ascii  = Common::ASCII_RETURN;
			
			etat = -1;
			cpt = 0;
			return true;
	}
	//END
	
	//-----------------------
	//END
	
	int eventType;
	float xUnit, yUnit;
	
	//HACK TEMPO
	if( bTempo != -1 && bTempo < 5 )
	{ bTempo++; return false; }
	else if( bTempo == 5 )
	{ bTempo = -1; return true; }
	//END
	
	//HACK GO
	if( idLvLCourant == 8 )
	{
		if( bGo != -1 && bGo < 25 )
		{ bGo++; return false; }
		else if( bGo == 25 )
		{ bGo = -1; [[NSNotificationCenter defaultCenter] postNotificationName:@"goMsgNotif" object:nil]; return true; }
	}
	else
	{
		if( bGo != -1 && bGo < 17 )
		{ bGo++; return false; }
		else if( bGo == 17 )
		{ bGo = -1; [[NSNotificationCenter defaultCenter] postNotificationName:@"goMsgNotif" object:nil]; return true; }	
	}
	//END
	
	//SIMULATION CLIC GAUCHE ET DROIT POUR GESTION ACTION
	if( bAction == 5 ) //si transition, on enchaine avec clic gauche et on redemarre
	{
		event.type = Common::EVENT_LBUTTONDOWN;
		event.mouse.x = _mouseX;
		event.mouse.y = _mouseY;
		bAction = 0;
		bLeftClick = true;
		bRightClick = false;
		return true;
	}
	else if( bAction != -1 &&  bAction < 3 ) //on boucle - cpt de frames
	{
		bAction++;
		return false;
	}
	else if( bAction == 3 ) //si ready
	{
		bAction = -1;
		if( bLeftClick ) //on regarde quel clic il s'agit
		{			
			event.type = Common::EVENT_LBUTTONUP;
			event.mouse.x = _mouseX;
			event.mouse.y = _mouseY;
			bLeftClick = false;
			
			if( bRightLeftClick )
			{
				bAction = 0;
				bRightLeftClick = false;
			}
		}
		else if( bRightClick )
		{
			event.type = Common::EVENT_RBUTTONUP;
			event.mouse.x = _mouseX;
			event.mouse.y = _mouseY;
			bRightClick = false;
			
			if( bRightLeftClick )
				bAction = 0;
		}
		else if( bRightLeftClick )//si pas de clic, transition des 2 clic (clic droit + clic gauche)
		{
			bAction = 5;
		}
		return true;
	}
	
	//HACK PAUSE
	if( bPause != -1 && bPause < 3 )
	{
		bPause++;
		return false;
	}
	else if( bPause == 3 )
	{
		bPause = -1;
		event.type = Common::EVENT_KEYUP;
		event.kbd.flags = 0;
		event.kbd.keycode = Common::KEYCODE_ESCAPE;
		event.kbd.ascii  = Common::ASCII_ESCAPE;
		bPauseActive = false;		
		return true;
	}
	
	//FETCH EVENT iPhone
	if (iPhone_fetchEvent(&eventType, &xUnit, &yUnit))
	{		
		int x = 0;
		int y = 0;
		xUnit = xUnit / 480.0f;
		yUnit = yUnit / 320.0f;
		switch (_screenOrientation)
		{
			case kScreenOrientationFlippedLandscape:
			{
				y = (int)(yUnit * _screenHeight);
				x = (int)(xUnit * _screenWidth);
			}
				break;
		}
		
		switch ((InputEvent)eventType)
		{
			case kButtonLeftDown:
				event.type = Common::EVENT_LBUTTONDOWN;
				event.mouse.x = _mouseX;
				event.mouse.y = _mouseY;
				bAction = 0;
				bLeftClick = true;
				bRightClick = false;
				bRightLeftClick = false;
				return true;
			case kButtonLeftUp:
				event.type = Common::EVENT_LBUTTONUP;
				event.mouse.x = _mouseX;
				event.mouse.y = _mouseY;
				return true;
			case kButtonRightDown:
				event.type = Common::EVENT_RBUTTONDOWN;
				event.mouse.x = _mouseX;
				event.mouse.y = _mouseY;
				bAction = 0;
				bLeftClick = false;
				bRightClick = true;
				bRightLeftClick = false;
				return true;
			case kButtonRightUp:
				event.type = Common::EVENT_RBUTTONUP;
				event.mouse.x = _mouseX;
				event.mouse.y = _mouseY;
				return true;
			case kButtonRightLeft:
				event.type = Common::EVENT_RBUTTONDOWN;
				event.mouse.x = _mouseX;
				event.mouse.y = _mouseY;
				bAction = 0;
				bLeftClick = false;
				bRightClick = true;
				bRightLeftClick = true;
				return true;
			case kTempo:
				bTempo = 0;
				return true;
			case kGo:
				bGo = 0;
				return true;				
			case kButtonEscapeDown:
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_ESCAPE;
				event.kbd.ascii  = Common::ASCII_ESCAPE;
				bPauseActive = true;
				break;
			case kButtonEscapeUp:
				event.type = Common::EVENT_KEYUP;
				event.kbd.flags = 0;
				event.kbd.keycode =  Common::KEYCODE_ESCAPE;
				event.kbd.ascii =  Common::ASCII_ESCAPE;
				bPauseActive = false;				
				break;	
			case kButtonPause:
				event.type = Common::EVENT_KEYDOWN;
				event.kbd.flags = 0;
				event.kbd.keycode = Common::KEYCODE_ESCAPE;
				event.kbd.ascii  = Common::ASCII_ESCAPE;
				bPause = 0;
				bPauseActive = true;				
				return true;	
			case kButtonCode:
				etat++;
				break;	
			case kInputMouseDragged:
				if (!handleEvent_mouseDragged(event, x, y))
					return false;	
			case kInputMouseDown:
				if (!handleEvent_mouseDown(event, x, y))
					return false;
				break;
			case kInputMouseUp:
				if (!handleEvent_mouseUp(event, x, y))
					return false;
				break;		
			default:
				break;
		}

		delayMillis(10);
		return true;
	}
	return false;
}

bool OSystem_IPHONE::preSimulation()
{
	//WAIT FRAMES TO LAUNCH SIMUL HACK
	if( strcmp( niveau, "AVT003.AVJ" ) == 0 && !bSimulCode && bSimulation )
	{
		if( bCptSimul == 10 )
		{
			etat = 0;
			bCptSimul = 0;
			bSimulCode = true;	
			return true;
		}
		else
		{
			bCptSimul++;
			return false;
		}
	}
	else if( bLvLOneLoaded && bSimulation ) //remplacer par bSimulation
	{
		if( bCptSimul == 20 )
		{
			etat = 0;
			bLvLOneLoaded = false;			
			bCptSimul = 0;
			return true;
		}
		else
		{
			bCptSimul++;
			return false;
		}
	}
	//END
	
	return true;
}

bool OSystem_IPHONE::handleEvent_mouseDown(Common::Event &event, int x, int y)
{
	//printf("Mouse down at (%u, %u)\n", x, y);

	// Workaround: kInputMouseSecondToggled isn't always sent when the
	// secondary finger is lifted. Need to make sure we get out of that mode.
	_secondaryTapped = false;

	if (_touchpadModeEnabled) 
	{
		_lastPadX = x;
		_lastPadY = y;
	} 
	else
		warpMouse(x, y);

	_lastMouseDown = getMillis();
	return false;
}

bool OSystem_IPHONE::handleEvent_mouseUp(Common::Event &event, int x, int y)
{
	//printf("Mouse up at (%u, %u)\n", x, y);

	if (_secondaryTapped)
	{
		_secondaryTapped = false;
		//if (!handleEvent_secondMouseUp(event, x, y))
			//return false;
	}
	else 
	{
		if (getMillis() - _lastMouseDown < 250)
		{
			event.type = Common::EVENT_LBUTTONDOWN;
			event.mouse.x = _mouseX;
			event.mouse.y = _mouseY;

			_queuedInputEvent.type = Common::EVENT_LBUTTONUP;
			_queuedInputEvent.mouse.x = _mouseX;
			_queuedInputEvent.mouse.y = _mouseY;
			_lastMouseTap = getMillis();
			_needEventRestPeriod = true;
		} 
		else
			return false;
	}

	return true;
}

bool OSystem_IPHONE::handleEvent_secondMouseDown(Common::Event &event, int x, int y) {
	_lastSecondaryDown = getMillis();
	_gestureStartX = x;
	_gestureStartY = y;

	return false; // or true ??
}

bool OSystem_IPHONE::handleEvent_secondMouseUp(Common::Event &event, int x, int y) {
	int curTime = getMillis();

	if (curTime - _lastSecondaryDown < 400 ) {
		//printf("Right tap!\n");
		if (curTime - _lastSecondaryTap < 400 && !_overlayVisible) {
			//printf("Right escape!\n");
			event.type = Common::EVENT_KEYDOWN;
			_queuedInputEvent.type = Common::EVENT_KEYUP;

			event.kbd.flags = _queuedInputEvent.kbd.flags = 0;
			event.kbd.keycode = _queuedInputEvent.kbd.keycode = Common::KEYCODE_ESCAPE;
			event.kbd.ascii = _queuedInputEvent.kbd.ascii = Common::ASCII_ESCAPE;
			_needEventRestPeriod = true;
			_lastSecondaryTap = 0;
		} 
		else 
		{
			//printf("Rightclick!\n");
			event.type = Common::EVENT_RBUTTONDOWN;
			event.mouse.x = _mouseX;
			event.mouse.y = _mouseY;
			_queuedInputEvent.type = Common::EVENT_RBUTTONUP;
			_queuedInputEvent.mouse.x = _mouseX;
			_queuedInputEvent.mouse.y = _mouseY;
			_lastSecondaryTap = curTime;
			_needEventRestPeriod = true;
		} 
	}
	return true;
}

bool OSystem_IPHONE::handleEvent_mouseDragged(Common::Event &event, int x, int y)
{
	if (_lastDragPosX == x && _lastDragPosY == y)
		return false;

	_lastDragPosX = x;
	_lastDragPosY = y;

	//printf("Mouse dragged at (%u, %u)\n", x, y);
	int mouseNewPosX;
	int mouseNewPosY;

	if (_touchpadModeEnabled )
	{
		int deltaX = _lastPadX - x;
		int deltaY = _lastPadY - y;
		_lastPadX = x;
		_lastPadY = y;

		mouseNewPosX = (int)(_mouseX - deltaX / 0.5f);
		mouseNewPosY = (int)(_mouseY - deltaY / 0.5f);

		if (mouseNewPosX < 0)
			mouseNewPosX = 0;
		else if (mouseNewPosX > _screenWidth)
			mouseNewPosX = _screenWidth;

		if (mouseNewPosY < 0)
			mouseNewPosY = 0;
		else if (mouseNewPosY > _screenHeight)
			mouseNewPosY = _screenHeight;

	} 
	else 
	{
		mouseNewPosX = x;
		mouseNewPosY = y;
	}

	event.type = Common::EVENT_MOUSEMOVE;
	event.mouse.x = mouseNewPosX;
	event.mouse.y = mouseNewPosY;
	warpMouse(mouseNewPosX, mouseNewPosY);

	return true;
}

bool OSystem_IPHONE::handleEvent_mouseSecondDragged(Common::Event &event, int x, int y) {
	if (_gestureStartX == -1 || _gestureStartY == -1) {
		return false;
	}

	static const int kNeededLength = 100;
	static const int kMaxDeviation = 20;

	int vecX = (x - _gestureStartX);
	int vecY = (y - _gestureStartY);
	
	int absX = abs(vecX);
	int absY = abs(vecY);

	//printf("(%d, %d)\n", vecX, vecY);

	if (absX >= kNeededLength || absY >= kNeededLength) { // Long enough gesture to react upon.
		_gestureStartX = -1;
		_gestureStartY = -1;
	}

	return false;
}